/******************************************************************************
 * $Id: gfx_types.cc,v 1.1 2006/10/29 23:15:11 devel Exp $
 * @file
 * @author	Jim E. Brooks  http://www.jimbrooks.org
 * @brief	Basic types.
 *//*
 * LEGAL:	COPYRIGHT (C) 2006 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE (GPL).
 ******************************************************************************/

#define GFX_TYPES_CC 1
#include "gfx.hh"
#include "gfx_stream.hh"
namespace gfx {

////////////////////////////////////////////////////////////////////////////////
//////////////////////////////  Multivar  //////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
 * Clear all values.
 *****************************************************************************/
void
Multivar::Clear( void )
{
    mValues.clear();
}

/*****************************************************************************
 * Remove value.
 *****************************************************************************/
void
Multivar::Remove( const int value )
{
    mValues.remove( value );
}

/*****************************************************************************
 * Replace all values with this value (replacement).
 *****************************************************************************/
void
Multivar::Set( const int value )
{
    mValues.clear();
    mValues.push_back( value );
}

/*****************************************************************************
 * Add an additional value (additive).
 *****************************************************************************/
void
Multivar::Add( const int value )
{
    // Add value unless present.
    if ( ! Query(value) )
		mValues.push_back( value );
}

/*****************************************************************************
 * Add or remove value.
 *****************************************************************************/
void
Multivar::Toggle( const int value )
{
    if ( Query(value) )
		Remove( value );
	else
		Add( value );
}

/*****************************************************************************
 * @return True if variable has this value.
 *****************************************************************************/
bool
Multivar::Query( const int value ) const
{
    return std::find( mValues.begin(), mValues.end(), value ) != mValues.end();
}

} // namespace gfx
