/*****************************************************************************
 * $Id: eng_python.cc,v 1.12 2006/10/10 04:14:56 devel Exp $
 * @file
 * @author	Jim E. Brooks  http://www.jimbrooks.org
 * @brief	Python support.
 * @remarks
 *//*
 * LEGAL:	COPYRIGHT (C) 2005 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE (GPL).
 *****************************************************************************/

#ifndef NO_PYTHON

#define ENG_PYTHON_CC 1
#include "gfx.hh"
#include "gfx_shared_val.hh"
#include "gfx_stream.hh"
#include "eng.hh"
#include "eng_python.hh"
#include "eng_python_ifc.hh"

////////////////////////////////////////////////////////////////////////////////
////////////////////////////  Python class /////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
 * Start Python interpreter.
 *****************************************************************************/
bool
Python::Init( void )
{
	// Initialize Python interpreter.
	Py_Initialize();
	if ( Py_IsInitialized() )
	{
#if PYTHON_THREADS
		// Initialize Python thread support.
		PyEval_InitThreads();
#endif
		return true;
	}
	else
	{
		PRINT_PYTHON_ERROR( "Py_IsInitialized() failed" );
		return false;
    }
}

/*****************************************************************************
 * Stop Python interpreter.
 *****************************************************************************/
void
Python::Exit( void )
{
	Py_Finalize();
#if PYTHON_THREADS
#error "TBD"
#endif
}

/*****************************************************************************
 * Create a new Python module (just an empty module).
 * @return PyObject or NULL.
 * @param	moduleName
 *          Name to give to Python module.
 * @remarks Returned PyObject is a borrowed reference (don't DEC it).
 *****************************************************************************/
PyObject*
Python::NewModule( const char* moduleName )
{
	static PyMethodDef methods[] = { {NULL,NULL,0,NULL} };
	return NewModule( moduleName, methods );
}

/*****************************************************************************
 * Create a new Python module implemented as C/C++ code.
 * C/C++ functions will be imported as Python functions.
 * @return PyObject or NULL.
 * @param	moduleName
 *          Name to give to Python module.
 * @param	methods
 *          Table of C functions to import to Python as Python functions.
 * @remarks Returned PyObject is a borrowed reference (don't DEC it).
 *****************************************************************************/
PyObject*
Python::NewModule( const char* moduleName, PyMethodDef methods[] )
{
	return Py_InitModule( const_cast<char*>(moduleName), methods );
}

/*****************************************************************************
 * Execute Python code in a C string.
 *****************************************************************************/
bool
Python::Run( const char* pyCode )
{
    // int PyRun_SimpleString ( char *command )
    // Executes the Python source code from command in the __main__ module.
	// If __main__ does not already exist, it is created.
	// Returns 0 on success or -1 if an exception was raised.

	return PyRun_SimpleString( const_cast<char*>(pyCode) ) == 0;
}
bool
Python::Run( const string& pyCode )
{
	return Run( pyCode.c_str() );
}

////////////////////////////////////////////////////////////////////////////////
/////////////////////////  3D ObjectPython class  //////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
 * Constructor.
 *****************************************************************************/
ObjectPython::ObjectPython( QuadrantPtrConst q )
	: Parent(q)
{
	Init();
	DEBUG_CODE( CheckObjectCtor(); )
}
ObjectPython::~ObjectPython()
{
	INVALIDATE_TYPESIG( this, TYPESIG_OBJECT_PYTHON );
	DEBUG_CODE( CheckObjectDtor(); )
}

/*****************************************************************************
 * Initialize.
 *****************************************************************************/
void
ObjectPython::Init( void )
{
	mVertexCnt		= 0;
	mWorldVertexs	= NULL;
	SET_TYPESIG( this, TYPESIG_OBJECT_PYTHON );
}

/*****************************************************************************
 * Draw.
 *****************************************************************************/
void
ObjectPython::Draw( void )
{
	GetPolygonList().Draw( this );
}

////////////////////////////////////////////////////////////////////////////////
/////////////////////////  3D FixedPython class  ///////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
 * ctor/dtor
 *****************************************************************************/
FixedPython::FixedPython( QuadrantPtrConst q, const Position& pos )
	: Parent(q,pos)
{
	DEBUG_CODE( CheckObjectCtor(); )
}
FixedPython::~FixedPython()
{
	DEBUG_CODE( CheckObjectDtor(); )
}

#endif // NO_PYTHON
